@@ -1,3 +1,6 @@
+/** \file fetchmail.h  header file for fetchmail */
+#ifndef _FETCHMAIL_H
+#define _FETCHMAIL_H
 /*
  * For license terms, see the file COPYING in this directory.
  */
@@ -5,9 +8,42 @@
 /* We need this for HAVE_STDARG_H, etc */
 #include "config.h"
 
+struct addrinfo;
+
 /* We need this for size_t */
 #include <sys/types.h>
 
+/* We need this for time_t */
+#if TIME_WITH_SYS_TIME
+# include <sys/time.h>
+# include <time.h>
+#else
+# if HAVE_SYS_TIME_H
+#  include <sys/time.h>
+# else
+#  include <time.h>
+# endif
+#endif
+
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#endif
+#ifdef HAVE_NET_SOCKET_H
+#include <net/socket.h>
+#endif
+#include <netdb.h>
+#include <stdio.h>
+
+/* Import Trio if needed */
+#if !defined(HAVE_SNPRINTF) || !defined(HAVE_VSNPRINTF)
+#  include "trio/trio.h"
+#endif
+
+/* We need this for strstr */
+#if !defined(HAVE_STRSTR) && !defined(strstr)
+char *strstr(const char *, const char *);
+#endif
+
 /* constants designating the various supported protocols */
 #define		P_AUTO		1
 #define		P_POP2		2
@@ -18,13 +54,9 @@
 #define		P_ETRN		7
 #define		P_ODMR		8
 
-#if INET6_ENABLE
 #define		SMTP_PORT	"smtp"
+#define		SMTP_PORT_NUM	25
 #define		KPOP_PORT	"kpop"
-#else /* INET6_ENABLE */
-#define		SMTP_PORT	25
-#define		KPOP_PORT	1109
-#endif /* INET6_ENABLE */
 
 #ifdef SSL_ENABLE
 #define		SIMAP_PORT	993
@@ -48,9 +80,19 @@
 #define		A_KERBEROS_V5	6	/* authenticate w/ Kerberos V5 */
 #define 	A_GSSAPI	7	/* authenticate with GSSAPI */
 #define		A_SSH		8	/* authentication at session level */
+#define		A_MSN		9	/* same as NTLM with keyword MSN */
+#define		A_EXTERNAL	10	/* external authentication (client cert) */
 
-/* some protocols (KERBEROS, GSSAPI, SSH) don't require a password */
-#define NO_PASSWORD(ctl)	((ctl)->server.authenticate > A_OTP || (ctl)->server.protocol == P_ETRN)
+/* some protocols or authentication types (KERBEROS, GSSAPI, SSH) don't
+ * require a password */
+#define NO_PASSWORD(ctl) \
+    ((ctl)->server.authenticate == A_OTP \
+     || (ctl)->server.authenticate == A_KERBEROS_V4 \
+     || (ctl)->server.authenticate == A_KERBEROS_V5 \
+     || (ctl)->server.authenticate == A_GSSAPI \
+     || (ctl)->server.authenticate == A_SSH \
+     || (ctl)->server.authenticate == A_EXTERNAL \
+     || (ctl)->server.protocol == P_ETRN)
 
 /*
  * Definitions for buffer sizes.  We get little help on setting maxima
@@ -79,6 +121,8 @@
 #define		DIGESTLEN	33	/* length of MD5 digest */
 
 /* exit code values */
+/* NOTE THAT PS_SUCCESS MUST ALWAYS BE 0 - SOME PARTS OF THE CODE
+ * RELY ON THIS VALUE! */
 #define		PS_SUCCESS	0	/* successful receipt of messages */
 #define		PS_NOMAIL       1	/* no mail available */
 #define		PS_SOCKET	2	/* socket I/O woes */
@@ -94,14 +138,13 @@
 #define		PS_BSMTP	12	/* output batch could not be opened */
 #define		PS_MAXFETCH	13	/* poll ended by fetch limit */
 #define		PS_SERVBUSY	14	/* server is busy */
-#define		PS_IDLETIMEOUT	15	/* timeout on imap IDLE */
 /* leave space for more codes */
 #define		PS_UNDEFINED	23	/* something I hadn't thought of */
 #define		PS_TRANSIENT	24	/* transient failure (internal use) */
 #define		PS_REFUSED	25	/* mail refused (internal use) */
 #define		PS_RETAINED	26	/* message retained (internal use) */
-#define		PS_TRUNCATED	27	/* headers incomplete (internal use) */
 #define		PS_REPOLL	28	/* repoll immediately with changed parameters (internal use) */
+#define		PS_IDLETIMEOUT	29	/* timeout on imap IDLE (internal use) */
 
 /* output noise level */
 #define         O_SILENT	0	/* mute, max squelch, etc. */
@@ -126,15 +169,17 @@ typedef	char	flag;
 #define FLAG_TRUE	2
 #define FLAG_FALSE	1
 
+/** run control data */
 struct runctl
 {
-    char	*logfile;
-    char	*idfile;
-    int		poll_interval;
+    char	*logfile;	/** where to write log information */
+    char	*idfile;	/** where to store UID data */
+    char	*pidfile;	/** where to record the PID of daemon mode processes */
     char	*postmaster;
+    char	*properties;
+    int		poll_interval;	/** poll interval in seconds (daemon mode, 0 == off) */
     flag	bouncemail;
     flag	spambounce;
-    char	*properties;
     flag	use_syslog;
     flag	invisible;
     flag	showdots;
@@ -142,20 +187,20 @@ struct runctl
 
 struct idlist
 {
-    unsigned char *id;
+    char *id;
     union
     {
 	struct
 	{
-	    short	num;
+	    int		num;
 	    flag	mark;		/* UID-index information */
 #define UID_UNSEEN	0		/* hasn't been seen */
 #define UID_SEEN	1		/* seen, but not deleted */
-#define UID_DELETED	2		/* this message has been deleted */
-#define UID_EXPUNGED	3		/* this message has been expunged */ 
+#define UID_DELETED	2		/* this message has been marked deleted */
+#define UID_EXPUNGED	3		/* this message has been expunged */
         }
 	status;
-	unsigned char *id2;
+	char *id2;
     } val;
     struct idlist *next;
 };
@@ -165,13 +210,8 @@ struct query;
 struct method		/* describe methods for protocol state machine */
 {
     const char *name;		/* protocol name */
-#if INET6_ENABLE
-    const char *service;
-    const char *sslservice;
-#else /* INET6_ENABLE */
-    int	port;			/* service port */
-    int	sslport;		/* service port for ssl */
-#endif /* INET6_ENABLE */
+    const char *service;	/* service port (unencrypted) */
+    const char *sslservice;	/* service port (SSL) */
     flag tagged;		/* if true, generate & expect command tags */
     flag delimited;		/* if true, accept "." message delimiter */
     int (*parse_response)(int, char *);
@@ -182,18 +222,22 @@ struct method		/* describe methods for protocol state machine */
 				/* get message range to fetch */
     int (*getsizes)(int, int, int *);
 				/* get sizes of messages */
+    int (*getpartialsizes)(int, int, int, int *);
+				/* get sizes of subset of messages */
     int (*is_old)(int, struct query *, int);
 				/* check for old message */
     int (*fetch_headers)(int, struct query *, int, int *);
-				/* fetch FROM headera given message */
+				/* fetch header from a given message */
     int (*fetch_body)(int, struct query *, int, int *);
 				/* fetch a given message */
-    int (*trail)(int, struct query *, int);
+    int (*trail)(int, struct query *, const char *);
 				/* eat trailer of a message */
-    int (*delete)(int, struct query *, int);
+    int (*delete_msg)(int, struct query *, int);
 				/* delete method */
     int (*mark_seen)(int, struct query *, int);
 				/* mark as seen method */
+    int (*end_mailbox_poll)(int, struct query *);
+				/* end-of-mailbox processing */
     int (*logout_cmd)(int, struct query *);
 				/* logout command */
     flag retry;			/* can getrange poll for new messages? */
@@ -207,12 +251,7 @@ struct hostdata		/* shared among all user connections to given server */
     struct idlist *akalist;		/* server name first, then akas */
     struct idlist *localdomains;	/* list of pass-through domains */
     int protocol;			/* protocol type */
-#if INET6_ENABLE
-    char *service;			/* IPv6 service name */
-    void *netsec;			/* IPv6 security request */
-#else /* INET6_ENABLE */
-    int port;				/* TCP/IP service port number */
-#endif /* INET6_ENABLE */
+    char *service;			/* service name */
     int interval;			/* # cycles to skip between polls */
     int authenticate;			/* authentication mode to try */
     int timeout;			/* inactivity timout in seconds */
@@ -225,16 +264,21 @@ struct hostdata		/* shared among all user connections to given server */
 #ifdef SDPS_ENABLE
     flag sdps;				/* use Demon Internet SDPS *ENV */
 #endif /* SDPS_ENABLE */
-    flag checkalias;                  	/* resolve aliases by comparing IPs? */
+    flag checkalias;			/* resolve aliases by comparing IPs? */
+    flag tracepolls;			/* if TRUE, add poll trace info to Received */
     char *principal;			/* Kerberos principal for mail service */
     char *esmtp_name, *esmtp_password;	/* ESMTP AUTH information */
 
 #if defined(linux) || defined(__FreeBSD__)
+#define CAN_MONITOR
+#endif
+
+#ifdef CAN_MONITOR
     char *interface;
     char *monitor;
     int  monitor_io;
     struct interface_pair_s *interface_pair;
-#endif /* linux */
+#endif
 
     char *plugin,*plugout;
 
@@ -243,11 +287,20 @@ struct hostdata		/* shared among all user connections to given server */
     int poll_count;			/* count of polls so far */
     char *queryname;			/* name to attempt DNS lookup on */
     char *truename;			/* "true name" of server host */
-    char *trueaddr;                     /* IP address of truename, as char */
+    struct sockaddr *trueaddr;		/* IP address of truename */
+    size_t trueaddr_len;		/* size of trueaddr data */
     struct hostdata *lead_server;	/* ptr to lead query for this server */
     int esmtp_options;
+    int workarounds;			/* track which workarounds the user was warned about */
 };
 
+/*
+ * bit flags to set in workarounds after the corresponding warning,
+ * which we assume to be server-specific, has been printed,
+ * so we don't spam our users in daemon mode.
+ */
+#define WKA_TOP (1L << 0)		/* Maillennium TOP -> RETR override warning */
+
 struct query
 {
     /* mailserver connection controls */
@@ -278,6 +331,7 @@ struct query
     flag keep;			/* if TRUE, leave messages undeleted */
     flag fetchall;		/* if TRUE, fetch all (not just unseen) */
     flag flush;			/* if TRUE, delete messages already seen */
+    flag limitflush;		/* if TRUE, delete oversized mails */
     flag rewrite;		/* if TRUE, canonicalize recipient addresses */
     flag stripcr;		/* if TRUE, strip CRs in text */
     flag forcecr;		/* if TRUE, force CRs before LFs in text */
@@ -289,38 +343,42 @@ struct query
     int	limit;			/* limit size of retrieved messages */
     int warnings;		/* size warning interval */
     int	fetchlimit;		/* max # msgs to get in single poll */
+    int fetchsizelimit;		/* max # msg sizes to get in a request */
+    int fastuidl;		/* do binary search for new UIDLs? */
+    int fastuidlcount;		/* internal count for frequency of binary search */
     int	batchlimit;		/* max # msgs to pass in single SMTP session */
     int	expunge;		/* max # msgs to pass between expunges */
     flag use_ssl;		/* use SSL encrypted session */
     char *sslkey;		/* optional SSL private key file */
     char *sslcert;		/* optional SSL certificate file */
-	char *sslproto;		/* force usage of protocol (ssl2|ssl3|tls1) - defaults to ssl23 */
+    char *sslproto;		/** force transport protocol (ssl2|ssl3|ssl23|tls1) - if NULL,
+				  use ssl23 for SSL and opportunistic tls1 for non-SSL connections. */
     char *sslcertpath;		/* Trusted certificate directory for checking the server cert */
     flag sslcertck;		/* Strictly check the server cert. */
     char *sslfingerprint;	/* Fingerprint to check against */
     char *properties;		/* passthrough properties for extensions */
-    flag tracepolls;		/* if TRUE, add poll trace info to Received */
 
     /* internal use -- per-poll state */
     flag active;		/* should we actually poll this server? */
-    const char *destaddr;	/* destination host for this query */
+    char *destaddr;		/* destination host for this query */
     int errcount;		/* count transient errors in last pass */
     int authfailcount;		/* count of authorization failures */
     int wehaveauthed;		/* We've managed to logon at least once! */
     int wehavesentauthnote;	/* We've sent an authorization failure note */
     int wedged;			/* wedged by auth failures or timeouts? */
     char *smtphost;		/* actual SMTP host we connected to */
+    char smtphostmode;		/* what's the actual SMTP host's wire protocol? */
     int smtp_socket;		/* socket descriptor for SMTP connection */
     unsigned int uid;		/* UID of user to deliver to */
     struct idlist *skipped;	/* messages skipped on the mail server */
     struct idlist *oldsaved, *newsaved;
     struct idlist **oldsavedend;
-    char *lastid;		/* last Message-ID seen on this connection */
-    char *thisid;		/* Message-ID of current message */
+    char lastdigest[DIGESTLEN];	/* last MD5 hash seen on this connection */
+    char *folder;		/* folder currently being polled */
 
     /* internal use -- per-message state */
     int mimemsg;		/* bitmask indicating MIME body-type */
-    char digest [DIGESTLEN];	/* md5 digest buffer */
+    char digest[DIGESTLEN];	/* md5 digest buffer */
 
     /* internal use -- housekeeping */
     struct query *next;		/* next query control block in chain */
@@ -380,6 +438,7 @@ extern flag peek_capable;	/* can we read msgs without setting seen? */
 extern struct runctl run;	/* global controls for this run */
 extern flag nodetach;		/* if TRUE, don't detach daemon process */
 extern flag quitmode;		/* if --quit was set */
+extern int  quitind;		/* optind after position of last --quit option */
 extern flag check_only;		/* if --check was set */
 extern char *rcfile;		/* path name of rc file */
 extern int linelimit;		/* limit # lines retrieved per site */
@@ -391,12 +450,17 @@ extern int pass;		/* number of re-polling pass */
 extern flag configdump;		/* dump control blocks as Python dictionary */
 extern char *fetchmailhost;	/* either "localhost" or an FQDN */
 extern int suppress_tags;	/* suppress tags in tagged protocols? */
-extern char shroud[PASSWORDLEN*2+1];	/* string to shroud in debug output */
+extern char shroud[PASSWORDLEN*2+3];	/* string to shroud in debug output */
 #ifdef SDPS_ENABLE
 extern char *sdps_envfrom;
 extern char *sdps_envto;
 #endif /* SDPS_ENABLE */
 
+extern const char *iana_charset;	/* IANA assigned charset name */
+
+/* from ucs/norm_charmap.c */
+const char *norm_charmap(const char *name);
+
 /* prototypes for globally callable functions */
 
 /* from /usr/include/sys/cdefs.h */
@@ -428,7 +492,8 @@ void report_at_line ();
 
 /* driver.c -- main driver loop */
 void set_timeout(int);
-int isidletimeout(void);
+int is_idletimeout(void);
+void resetidletimeout(void);
 int do_protocol(struct query *, const struct method *);
 
 /* transact.c: transaction support */
@@ -437,7 +502,8 @@ int readheaders(int sock,
 		       long fetchlen,
 		       long reallen,
 		       struct query *ctl,
-		int num);
+		       int num,
+		       flag *suppress_readbody);
 int readbody(int sock, struct query *ctl, flag forward, int len);
 #if defined(HAVE_STDARG_H)
 void gen_send(int sock, const char *, ... )
@@ -454,12 +520,6 @@ int gen_transact();
 #endif
 extern struct msgblk msgblk;
 
-/* lock.c: concurrency locking */
-void lock_setup(void), lock_assert(void);
-void lock_or_die(void), lock_release(void);
-int lock_state(void);
-void lock_dispose(void);
-
 /* use these to track what was happening when the nonresponse timer fired */
 #define GENERAL_WAIT	0	/* unknown wait type */
 #define OPEN_WAIT	1	/* waiting from mailserver open */
@@ -486,6 +546,7 @@ extern int mytimeout;
 
 /* idle.c */
 int interruptible_idle(int interval);
+extern volatile int lastsig;
 
 /* sink.c: forwarding */
 void smtp_close(struct query *, int);
@@ -495,10 +556,10 @@ int stuffline(struct query *, char *);
 int open_sink(struct query*, struct msgblk *, int*, int*);
 void release_sink(struct query *);
 int close_sink(struct query *, struct msgblk *, flag);
-int open_warning_by_mail(struct query *, struct msgblk *);
+int open_warning_by_mail(struct query *);
 #if defined(HAVE_STDARG_H)
-void stuff_warning(struct query *, const char *, ... )
-    __attribute__ ((format (printf, 2, 3)))
+void stuff_warning(const char *, struct query *, const char *, ... )
+    __attribute__ ((format (printf, 3, 4)))
     ;
 #else
 void stuff_warning();
@@ -506,10 +567,12 @@ void stuff_warning();
 void close_warning_by_mail(struct query *, struct msgblk *);
 
 /* rfc822.c: RFC822 header parsing */
-unsigned char *reply_hack(unsigned char *, const unsigned char *);
-unsigned char *nxtaddr(const unsigned char *);
+char *reply_hack(char *, const char *, size_t *);
+char *nxtaddr(const char *);
 
 /* uid.c: UID support */
+extern int dofastuidl;
+
 void initialize_saved_lists(struct query *, const char *);
 struct idlist *save_str(struct idlist **, const char *, flag);
 void free_str_list(struct idlist **);
@@ -517,17 +580,20 @@ struct idlist *copy_str_list(struct idlist *idl);
 void save_str_pair(struct idlist **, const char *, const char *);
 void free_str_pair_list(struct idlist **);
 int delete_str(struct idlist **, long);
-int str_in_list(struct idlist **, const char *, const flag);
+struct idlist *str_in_list(struct idlist **, const char *, const flag);
 int str_nr_in_list(struct idlist **, const char *);
 int str_nr_last_in_list(struct idlist **, const char *);
 void str_set_mark( struct idlist **, const char *, const flag);
 int count_list( struct idlist **idl );
 char *str_from_nr_list( struct idlist **idl, long number );
 char *str_find(struct idlist **, long);
+struct idlist *id_find(struct idlist **idl, long);
 char *idpair_find(struct idlist **, const char *);
 void append_str_list(struct idlist **, struct idlist **);
 void expunge_uids(struct query *);
 void uid_swap_lists(struct query *);
+void uid_discard_new_list(struct query *ctl);
+void uid_reset_num(struct query *ctl);
 void write_saved_lists(struct query *, const char *);
 
 /* rcfile_y.y */
@@ -535,17 +601,17 @@ int prc_parse_file(const char *, const flag);
 int prc_filecheck(const char *, const flag);
 
 /* base64.c */
-void to64frombits(unsigned char *, const unsigned char *, int);
-int from64tobits(char *, const char *, int maxlen);
+void to64frombits(char *, const void *, int);
+int from64tobits(void *, const char *, int maxlen);
 
 /* unmime.c */
 /* Bit-mask returned by MimeBodyType */
 #define MSG_IS_7BIT       0x01
 #define MSG_IS_8BIT       0x02
 #define MSG_NEEDS_DECODE  0x80
-extern void UnMimeHeader(unsigned char *buf);
-extern int  MimeBodyType(unsigned char *hdrs, int WantDecode);
-extern int  UnMimeBodyline(unsigned char **buf, flag delimited, flag issoftline);
+extern void UnMimeHeader(char *buf);
+extern int  MimeBodyType(char *hdrs, int WantDecode);
+extern int  UnMimeBodyline(char **buf, flag delimited, flag issoftline);
 
 /* interface.c */
 void interface_init(void);
@@ -561,23 +627,8 @@ int interface_approve(struct hostdata *, flag domonitor);
 #endif
 XMALLOCTYPE *xmalloc(size_t);
 XMALLOCTYPE *xrealloc(/*@null@*/ XMALLOCTYPE *, size_t);
+#define xfree(p) { if (p) { free(p); } (p) = 0; }
 char *xstrdup(const char *);
-#if defined(HAVE_ALLOCA_H)
-#include <alloca.h>
-#else
-#ifdef _AIX
- #pragma alloca
-#endif
-#endif
-#define	xalloca(ptr, t, n)	if (!(ptr = (t) alloca(n)))\
-       {report(stderr, GT_("alloca failed")); exit(PS_UNDEFINED);}
-#if 0
-/*
- * This is a hack to help xgettext which cannot find strings in
- * macro definitions like the one for xalloca above.
- */
-static char *dummy = gettext_noop("alloca failed");
-#endif
 
 /* protocol driver and methods */
 int doPOP2 (struct query *); 
@@ -589,7 +640,7 @@ int doODMR (struct query *);
 /* authentication functions */
 int do_cram_md5(int sock, char *command, struct query *ctl, char *strip);
 int do_rfc1731(int sock, char *command, char *truename);
-int do_gssauth(int sock, char *command, char *hostname, char *username);
+int do_gssauth(int sock, char *command, char *service, char *hostname, char *username);
 int do_otp(int sock, char *command, struct query *ctl);
 
 /* miscellanea */
@@ -600,23 +651,35 @@ extern char currentwd[1024], rcfiledir[1024];
 struct query *hostalloc(struct query *); 
 int parsecmdline (int, char **, struct runctl *, struct query *);
 char *prependdir (const char *, const char *);
-char *MD5Digest (unsigned char *);
-void hmac_md5 (unsigned char *, size_t, unsigned char *, size_t, unsigned char *, size_t);
-int POP3_auth_rpa(unsigned char *, unsigned char *, int socket);
+char *MD5Digest (unsigned const char *);
+void hmac_md5 (char *, size_t, char *, size_t, unsigned char *, size_t);
+int POP3_auth_rpa(char *, char *, int socket);
 typedef RETSIGTYPE (*SIGHANDLERTYPE) (int);
 void deal_with_sigchld(void);
 RETSIGTYPE null_signal_handler(int sig);
 SIGHANDLERTYPE set_signal_handler(int sig, SIGHANDLERTYPE handler);
-int daemonize(const char *, void (*)(int));
+int daemonize(const char *);
 char *fm_getpassword(char *);
 void escapes(const char *, char *);
 char *visbuf(const char *);
 const char *showproto(int);
 void dump_config(struct runctl *runp, struct query *querylist);
-int is_host_alias(const char *, struct query *);
-char *host_fqdn(void);
+int is_host_alias(const char *, struct query *, struct addrinfo **);
+
+extern struct addrinfo *ai0, *ai1;
+
+/** Try to obtain fully qualified hostname of current host. Exit with
+ * PS_DNS if \a required is true and there is a DNS error. Exit with
+ * PS_DNS if gethostname() fails, independent of the value of \a
+ * required.
+ * \return
+ * - fully qualified hostname if \a required is non-zero.
+ * - unqualified or fully qualified hostname if \a required is zero (0).
+ */
+char *host_fqdn(int required /** exit with PS_DNS if the name cannot be qualified */);
 char *rfc822timestamp(void);
-flag isafile(int);
+flag is_a_file(int);
+char *rfc2047e(const char*, const char *);
 
 void yyerror(const char *);
 int yylex(void);
@@ -634,11 +697,11 @@ void itimerthread(void*);
 #define chdir _chdir2
 #endif /* _EMX_ */
 
-# if HAVE_STRERROR
-#  ifndef strerror		/* On some systems, strerror is a macro */
-char *strerror ();
+#ifdef HAVE_STRERROR
+#  if !defined(strerror) && !defined(HAVE_DECL_STRERROR)	/* On some systems, strerror is a macro */
+char *strerror (int);
 #  endif
-# endif /* HAVE_STRERROR */
+#endif /* HAVE_STRERROR */
 
 #define STRING_DISABLED	(char *)-1
 #define STRING_DUMMY	""
@@ -649,6 +712,10 @@ char *strerror ();
 #endif
 #endif
 
+#ifndef HAVE_STPCPY
+char *stpcpy(char *, const char*);
+#endif
+
 #ifdef FETCHMAIL_DEBUG
 #define exit(e) do { \
        FILE *out; \
@@ -667,4 +734,41 @@ char *strerror ();
 #define ROOT_UID 0
 #endif /* __CYGWIN__ */
 
+extern int mailserver_socket_temp;
+extern char *program_name;
+
+/* POSIX space characters,
+ * <tab>;<newline>;<vertical-tab>;<form-feed>;<carriage-return>;<space> */
+#define POSIX_space "\t\n\v\f\r "
+
+/* strlcpy/strlcat prototypes */
+#ifndef HAVE_STRLCAT
+size_t
+strlcat(char *dst, const char *src, size_t siz);
+#endif
+#ifndef HAVE_STRLCPY
+size_t
+strlcpy(char *dst, const char *src, size_t siz);
+#endif
+
+/** Resolve the a TCP service name or a string containing only a decimal
+ * positive integer to a port number. Returns -1 for error. */
+int servport(const char *service);
+
+#ifndef HAVE_GETNAMEINFO
+# define NI_NUMERICHOST	1
+# define NI_NUMERICSERV	2
+# define NI_NOFQDN	4
+# define NI_NAMEREQD	8
+# define NI_DGRAM	16
+#endif
+
+int fm_getaddrinfo(const char *node, const char *serv, const struct addrinfo *hints, struct addrinfo **res);
+void fm_freeaddrinfo(struct addrinfo *ai);
+
+/* prototypes from tls.c */
+int maybe_tls(struct query *ctl);
+int must_tls(struct query *ctl);
+
+#endif
 /* fetchmail.h ends here */
