@@ -23,7 +23,13 @@
 #include  "i18n.h"
 
 #ifdef OPIE_ENABLE
+#ifdef __cplusplus
+extern "C" {
+#endif
 #include <opie.h>
+#ifdef __cplusplus
+}
+#endif
 #endif /* OPIE_ENABLE */
 
 /* global variables: please reinitialize them explicitly for proper
@@ -35,6 +41,7 @@ static char lastok[POPBUFSIZE+1];
 #endif /* OPIE_ENABLE */
 
 /* session variables initialized in capa_probe() or pop3_getauth() */
+flag done_capa = FALSE;
 #if defined(GSSAPI)
 flag has_gssapi = FALSE;
 #endif /* defined(GSSAPI) */
@@ -46,7 +53,7 @@ static flag has_cram = FALSE;
 flag has_otp = FALSE;
 #endif /* OPIE_ENABLE */
 #ifdef SSL_ENABLE
-static flag has_ssl = FALSE;
+static flag has_stls = FALSE;
 #endif /* SSL_ENABLE */
 
 /* mailbox variables initialized in pop3_getrange() */
@@ -96,7 +103,7 @@ static int do_pop3_ntlm(int sock, struct query *ctl,
 	dumpSmbNtlmAuthRequest(stdout, &request);
 
     memset(msgbuf,0,sizeof msgbuf);
-    to64frombits (msgbuf, (unsigned char*)&request, SmbLength(&request));
+    to64frombits (msgbuf, &request, SmbLength(&request));
   
     if (outlevel >= O_MONITOR)
 	report(stdout, "POP3> %s\n", msgbuf);
@@ -107,7 +114,7 @@ static int do_pop3_ntlm(int sock, struct query *ctl,
     if ((gen_recv(sock, msgbuf, sizeof msgbuf)))
 	return result;
   
-    len = from64tobits ((unsigned char*)&challenge, msgbuf, sizeof(msgbuf));
+    len = from64tobits (&challenge, msgbuf, sizeof(msgbuf));
     
     if (outlevel >= O_DEBUG)
 	dumpSmbNtlmAuthChallenge(stdout, &challenge);
@@ -118,7 +125,7 @@ static int do_pop3_ntlm(int sock, struct query *ctl,
 	dumpSmbNtlmAuthResponse(stdout, &response);
   
     memset(msgbuf,0,sizeof msgbuf);
-    to64frombits (msgbuf, (unsigned char*)&response, SmbLength(&response));
+    to64frombits (msgbuf, &response, SmbLength(&response));
 
     if (outlevel >= O_MONITOR)
 	report(stdout, "POP3> %s\n", msgbuf);
@@ -232,6 +239,9 @@ static int capa_probe(int sock)
 {
     int	ok;
 
+    if (done_capa) {
+	return PS_SUCCESS;
+    }
 #if defined(GSSAPI)
     has_gssapi = FALSE;
 #endif /* defined(GSSAPI) */
@@ -255,7 +265,7 @@ static int capa_probe(int sock)
 		break;
 #ifdef SSL_ENABLE
 	    if (strstr(buffer, "STLS"))
-		has_ssl = TRUE;
+		has_stls = TRUE;
 #endif /* SSL_ENABLE */
 #if defined(GSSAPI)
 	    if (strstr(buffer, "GSSAPI"))
@@ -273,6 +283,7 @@ static int capa_probe(int sock)
 		has_cram = TRUE;
 	}
     }
+    done_capa = TRUE;
     return(ok);
 }
 
@@ -296,9 +307,12 @@ static int pop3_getauth(int sock, struct query *ctl, char *greeting)
     char *challenge;
 #endif /* OPIE_ENABLE */
 #ifdef SSL_ENABLE
-    flag did_stls = FALSE;
+    char *realhost = ctl->server.via ? ctl->server.via : ctl->server.pollname;
+    flag connection_may_have_tls_errors = FALSE;
+    flag got_tls = FALSE;
 #endif /* SSL_ENABLE */
 
+    done_capa = FALSE;
 #if defined(GSSAPI)
     has_gssapi = FALSE;
 #endif /* defined(GSSAPI) */
@@ -310,7 +324,7 @@ static int pop3_getauth(int sock, struct query *ctl, char *greeting)
     has_otp = FALSE;
 #endif /* OPIE_ENABLE */
 #ifdef SSL_ENABLE
-    has_ssl = FALSE;
+    has_stls = FALSE;
 #endif /* SSL_ENABLE */
 
     /* Set this up before authentication quits early. */
@@ -398,25 +412,29 @@ static int pop3_getauth(int sock, struct query *ctl, char *greeting)
 
 	/*
 	 * CAPA command may return a list including available
-	 * authentication mechanisms.  if it doesn't, no harm done, we
-	 * just fall back to a plain login.  Note that this code 
-	 * latches the server's authentication type, so that in daemon mode
-	 * the CAPA check only needs to be done once at start of run.
+	 * authentication mechanisms and STLS capability.
 	 *
-	 * If CAPA fails, then force the authentication method to PASSORD
-	 * and repoll immediately.
+	 * If it doesn't, no harm done, we just fall back to a plain
+	 * login -- if the user allows it.
 	 *
-	 * These authentication methods are blessed by RFC1734,
-	 * describing the POP3 AUTHentication command.
+	 * Note that this code latches the server's authentication type,
+	 * so that in daemon mode the CAPA check only needs to be done
+	 * once at start of run.
+	 *
+	 * If CAPA fails, then force the authentication method to
+	 * PASSWORD, switch off opportunistic and repoll immediately.
+	 * If TLS is mandatory, fail up front.
 	 */
 	if ((ctl->server.authenticate == A_ANY) ||
-	    (ctl->server.authenticate == A_GSSAPI) ||
-	    (ctl->server.authenticate == A_KERBEROS_V4) ||
-	    (ctl->server.authenticate == A_OTP) ||
-	    (ctl->server.authenticate == A_CRAM_MD5))
+		(ctl->server.authenticate == A_GSSAPI) ||
+		(ctl->server.authenticate == A_KERBEROS_V4) ||
+		(ctl->server.authenticate == A_KERBEROS_V5) ||
+		(ctl->server.authenticate == A_OTP) ||
+		(ctl->server.authenticate == A_CRAM_MD5) ||
+		maybe_tls(ctl))
 	{
 	    if ((ok = capa_probe(sock)) != PS_SUCCESS)
-	    /* we are in STAGE_GETAUTH! */
+		/* we are in STAGE_GETAUTH => failure is PS_AUTHFAIL! */
 		if (ok == PS_AUTHFAIL ||
 		    /* Some servers directly close the socket. However, if we
 		     * have already authenticated before, then a previous CAPA
@@ -425,52 +443,89 @@ static int pop3_getauth(int sock, struct query *ctl, char *greeting)
 		     */
 		    (ok == PS_SOCKET && !ctl->wehaveauthed))
 		{
-		    ctl->server.authenticate = A_PASSWORD;
-		    /* repoll immediately */
-		    ok = PS_REPOLL;
-		    break;
+#ifdef SSL_ENABLE
+		    if (must_tls(ctl)) {
+			/* fail with mandatory STLS without repoll */
+			report(stderr, GT_("TLS is mandatory for this session, but server refused CAPA command.\n"));
+			report(stderr, GT_("The CAPA command is however necessary for TLS.\n"));
+			return ok;
+		    } else {
+			/* defeat opportunistic STLS */
+			xfree(ctl->sslproto);
+			ctl->sslproto = xstrdup("");
+		    }
+#endif
+		    /* If strong authentication was opportunistic, retry without, else fail. */
+		    switch (ctl->server.authenticate) {
+			case A_ANY:
+			    ctl->server.authenticate = A_PASSWORD;
+			    /* FALLTHROUGH */
+			case A_PASSWORD: /* this should only happen with TLS enabled */
+			    return PS_REPOLL;
+			default:
+			    return PS_AUTHFAIL;
+		    }
 		}
 	}
 
 #ifdef SSL_ENABLE
-	if (has_ssl
-	    && !ctl->use_ssl
-	    && (!ctl->sslproto || !strcmp(ctl->sslproto,"tls1")))
-	{
-	    char *realhost;
-
-	   realhost = ctl->server.via ? ctl->server.via : ctl->server.pollname;
-           ok = gen_transact(sock, "STLS");
-
-           /* We use "tls1" instead of ctl->sslproto, as we want STLS,
-            * not other SSL protocols
-            */
-	   if (ok == PS_SUCCESS &&
-	       SSLOpen(sock,ctl->sslcert,ctl->sslkey,"tls1",ctl->sslcertck, ctl->sslcertpath,ctl->sslfingerprint,realhost,ctl->server.pollname) == -1)
+	if (maybe_tls(ctl)) {
+	   if (has_stls)
 	   {
-	       if (!ctl->sslproto && !ctl->wehaveauthed)
+	       /* Use "tls1" rather than ctl->sslproto because tls1 is the only
+		* protocol that will work with STARTTLS.  Don't need to worry
+		* whether TLS is mandatory or opportunistic unless SSLOpen() fails
+		* (see below). */
+	       if (gen_transact(sock, "STLS") == PS_SUCCESS
+		       && SSLOpen(sock, ctl->sslcert, ctl->sslkey, "tls1", ctl->sslcertck,
+			   ctl->sslcertpath, ctl->sslfingerprint, realhost,
+			   ctl->server.pollname, &ctl->remotename) != -1)
 	       {
-		   ctl->sslproto = xstrdup("");
-		   /* repoll immediately */
-		   return(PS_REPOLL);
+		   /*
+		    * RFC 2595 says this:
+		    *
+		    * "Once TLS has been started, the client MUST discard cached
+		    * information about server capabilities and SHOULD re-issue the
+		    * CAPABILITY command.  This is necessary to protect against
+		    * man-in-the-middle attacks which alter the capabilities list prior
+		    * to STARTTLS.  The server MAY advertise different capabilities
+		    * after STARTTLS."
+		    *
+		    * Now that we're confident in our TLS connection we can
+		    * guarantee a secure capability re-probe.
+		    */
+		   got_tls = TRUE;
+		   done_capa = FALSE;
+		   ok = capa_probe(sock);
+		   if (ok != PS_SUCCESS) {
+		       return ok;
+		   }
+		   if (outlevel >= O_VERBOSE)
+		   {
+		       report(stdout, GT_("%s: upgrade to TLS succeeded.\n"), realhost);
+		   }
 	       }
-	       report(stderr,
-		       GT_("SSL connection failed.\n"));
-		return PS_SOCKET;
-	    }
-	   did_stls = TRUE;
+	   }
 
-	   /*
-	    * RFC 2595 says this:
-	    *
-	    * "Once TLS has been started, the client MUST discard cached
-	    * information about server capabilities and SHOULD re-issue the
-	    * CAPABILITY command.  This is necessary to protect against
-	    * man-in-the-middle attacks which alter the capabilities list prior
-	    * to STARTTLS.  The server MAY advertise different capabilities
-	    * after STARTTLS."
-	    */
-	   capa_probe(sock);
+	   if (!got_tls) {
+	       if (must_tls(ctl)) {
+		   /* Config required TLS but we couldn't guarantee it, so we must
+		    * stop. */
+		   report(stderr, GT_("%s: upgrade to TLS failed.\n"), realhost);
+		   return PS_SOCKET;
+	       } else {
+		   /* We don't know whether the connection is usable, and there's
+		    * no command we can reasonably issue to test it (NOOP isn't
+		    * allowed til post-authentication), so leave it in an unknown
+		    * state, mark it as such, and check more carefully if things
+		    * go wrong when we try to authenticate. */
+		   connection_may_have_tls_errors = TRUE;
+		   if (outlevel >= O_VERBOSE)
+		   {
+		       report(stdout, GT_("%s: opportunistic upgrade to TLS failed, trying to continue.\n"), realhost);
+		   }
+	       }
+	   }
 	}
 #endif /* SSL_ENABLE */
 
@@ -533,8 +588,10 @@ static int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	if ((challenge = strstr(lastok, "otp-"))) {
 	  char response[OPIE_RESPONSE_MAX+1];
 	  int i;
+	  char *n = xstrdup("");
 
-	  i = opiegenerator(challenge, !strcmp(ctl->password, "opie") ? "" : ctl->password, response);
+	  i = opiegenerator(challenge, !strcmp(ctl->password, "opie") ? n : ctl->password, response);
+	  free(n);
 	  if ((i == -2) && !run.poll_interval) {
 	    char secret[OPIE_SECRET_MAX+1];
 	    fprintf(stderr, GT_("Secret pass phrase: "));
@@ -553,16 +610,25 @@ static int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	}
 #endif /* OPIE_ENABLE */
 
-	strlcpy(shroud, ctl->password, sizeof(shroud));
-	ok = gen_transact(sock, "PASS %s", ctl->password);
+	/* check if we are actually allowed to send the password */
+	if (ctl->server.authenticate == A_ANY
+		|| ctl->server.authenticate == A_PASSWORD) {
+	    strlcpy(shroud, ctl->password, sizeof(shroud));
+	    ok = gen_transact(sock, "PASS %s", ctl->password);
+	} else {
+	    report(stderr, GT_("We've run out of allowed authenticators and cannot continue.\n"));
+	    ok = PS_AUTHFAIL;
+	}
+	memset(shroud, 0x55, sizeof(shroud));
 	shroud[0] = '\0';
 #ifdef SSL_ENABLE
 	/* this is for servers which claim to support TLS, but actually
 	 * don't! */
-	if (did_stls && ok == PS_SOCKET && !ctl->sslproto && !ctl->wehaveauthed)
+	if (connection_may_have_tls_errors && ok == PS_SOCKET)
 	{
+	    xfree(ctl->sslproto);
 	    ctl->sslproto = xstrdup("");
-	    /* repoll immediately */
+	    /* repoll immediately without TLS */
 	    ok = PS_REPOLL;
 	}
 #endif
@@ -601,8 +667,12 @@ static int pop3_getauth(int sock, struct query *ctl, char *greeting)
 	break;
 
     case P_RPOP:
-	if ((ok = gen_transact(sock,"USER %s", ctl->remotename)) == 0)
+	if ((ok = gen_transact(sock,"USER %s", ctl->remotename)) == 0) {
+	    strlcpy(shroud, ctl->password, sizeof(shroud));
 	    ok = gen_transact(sock, "RPOP %s", ctl->password);
+	    memset(shroud, 0x55, sizeof(shroud));
+	    shroud[0] = '\0';
+	}
 	break;
 
     default:
@@ -645,6 +715,13 @@ static void trim(char *s) {
     s[0] = '\0';
 }
 
+/* XXX FIXME: using the Message-ID is unsafe, some messages (spam,
+ * broken messages) do not have Message-ID headers, and messages without
+ * those appear to break this code and cause fetchmail (at least version
+ * 6.2.3) to not delete such messages properly after retrieval.
+ * See Sourceforge Bug #780933.
+ *
+ * The other problem is that the TOP command itself is optional, too... */
 static int pop3_gettopid(int sock, int num , char *id, size_t idsize)
 {
     int ok;
@@ -666,6 +743,8 @@ static int pop3_gettopid(int sock, int num , char *id, size_t idsize)
 	    trim(id);
 	}
     }
+    /* XXX FIXME: do not return success here if no Message-ID header was
+     * found. */
     return 0;
 }
 
@@ -706,7 +785,7 @@ static int pop3_getuidl(int sock, int num, char *id /** output */, size_t idsize
 	return(ok);
     if ((ok = parseuid(buf, &gotnum, id, idsize)))
 	return ok;
-    if (gotnum != num) {
+    if (gotnum != (unsigned long)num) {
 	report(stderr, GT_("Server responded with UID for wrong message.\n"));
 	return PS_PROTOCOL;
     }
@@ -723,20 +802,20 @@ static int pop3_fastuidl( int sock,  struct query *ctl, unsigned int count, int
     last_nr = count + 1;
     while (first_nr < last_nr - 1)
     {
-	struct idlist	*new;
+	struct idlist	*newl;
 
 	try_nr = (first_nr + last_nr) / 2;
 	if ((ok = pop3_getuidl(sock, try_nr, id, sizeof(id))) != 0)
 	    return ok;
-	if ((new = str_in_list(&ctl->oldsaved, id, FALSE)))
+	if ((newl = str_in_list(&ctl->oldsaved, id, FALSE)))
 	{
-	    flag mark = new->val.status.mark;
+	    flag mark = newl->val.status.mark;
 	    if (mark == UID_DELETED || mark == UID_EXPUNGED)
 	    {
 		if (outlevel >= O_VERBOSE)
 		    report(stderr, GT_("id=%s (num=%d) was deleted, but is still present!\n"), id, try_nr);
 		/* just mark it as seen now! */
-		new->val.status.mark = mark = UID_SEEN;
+		newl->val.status.mark = mark = UID_SEEN;
 	    }
 
 	    /* narrow the search region! */
@@ -750,7 +829,7 @@ static int pop3_fastuidl( int sock,  struct query *ctl, unsigned int count, int
 		first_nr = try_nr;
 
 	    /* save the number */
-	    new->val.status.num = try_nr;
+	    newl->val.status.num = try_nr;
 	}
 	else
 	{
@@ -759,8 +838,8 @@ static int pop3_fastuidl( int sock,  struct query *ctl, unsigned int count, int
 	    last_nr = try_nr;
 
 	    /* save it */
-	    new = save_str(&ctl->oldsaved, id, UID_UNSEEN);
-	    new->val.status.num = try_nr;
+	    newl = save_str(&ctl->oldsaved, id, UID_UNSEEN);
+	    newl->val.status.num = try_nr;
 	}
     }
     if (outlevel >= O_DEBUG && last_nr <= count)
@@ -774,6 +853,10 @@ static int pop3_fastuidl( int sock,  struct query *ctl, unsigned int count, int
 
 static int pop3_slowuidl( int sock,  struct query *ctl, int *countp, int *newp)
 {
+    /* XXX FIXME: this code is severely broken. A Cc:d mailing list
+     * message will arrive twice with the same Message-ID, so this
+     * slowuidl code will break. Same goes for messages without
+     * Message-ID headers at all. This code would best be removed. */
     /* This approach tries to get the message headers from the
      * remote hosts and compares the message-id to the already known
      * ones:
@@ -844,10 +927,10 @@ static int pop3_slowuidl( int sock,  struct query *ctl, int *countp, int *newp)
     /* the first try_id messages are known -> copy them to the newsaved list */
     for( num = first_nr; num < list_len; num++ )
     {
-	struct idlist	*new = save_str(&ctl->newsaved, 
+	struct idlist	*newl = save_str(&ctl->newsaved, 
 				str_from_nr_list(&ctl->oldsaved, num),
 				UID_UNSEEN);
-	new->val.status.num = num - first_nr + 1;
+	newl->val.status.num = num - first_nr + 1;
     }
 
     if( nolinear ) {
@@ -869,6 +952,7 @@ static int pop3_getrange(int sock,
     int ok;
     char buf [POPBUFSIZE+1];
 
+    (void)folder;
     /* Ensure that the new list is properly empty */
     ctl->newsaved = (struct idlist *)NULL;
 
@@ -888,13 +972,15 @@ static int pop3_getrange(int sock,
 	return(ok);
 
     /*
-     * Newer, RFC-1725-conformant POP servers may not have the LAST command.
-     * We work as hard as possible to hide this ugliness, but it makes
+     * Newer, RFC-1725/1939-conformant POP servers may not have the LAST
+     * command.  We work as hard as possible to hide this, but it makes
      * counting new messages intrinsically quadratic in the worst case.
      */
     last = 0;
     *newp = -1;
-    if (*countp > 0 && !ctl->fetchall)
+    /* if there are messages, and UIDL is desired, use UIDL
+     * also use UIDL if fetchall is unset */
+    if (*countp > 0 && (!ctl->fetchall || ctl->server.uidl))
     {
 	int fastuidl;
 	char id [IDLEN+1];
@@ -902,6 +988,7 @@ static int pop3_getrange(int sock,
 	/* should we do fast uidl this time? */
 	fastuidl = ctl->fastuidl;
 	if (*countp > 7 &&		/* linear search is better if there are few mails! */
+	    !ctl->fetchall &&		/* with fetchall, all uids are required */
 	    !ctl->flush &&		/* with flush, it is safer to disable fastuidl */
 	    NUM_NONZERO (fastuidl))
 	{
@@ -918,8 +1005,10 @@ static int pop3_getrange(int sock,
 	    ok = pop3_ok(sock, buf);
 	} else
 	    ok = 1;
+
 	if (ok == 0)
 	{
+	    /* scan LAST reply */
 	    if (sscanf(buf, "%d", &last) == 0)
 	    {
 		report(stderr, GT_("protocol error\n"));
@@ -929,6 +1018,7 @@ static int pop3_getrange(int sock,
 	}
 	else
 	{
+	    /* do UIDL */
 	    if (dofastuidl)
 		return(pop3_fastuidl( sock, ctl, *countp, newp));
 	    /* grab the mailbox's UID list */
@@ -943,6 +1033,7 @@ static int pop3_getrange(int sock,
 	    }
 	    else
 	    {
+		/* UIDL worked - parse reply */
 		unsigned long unum;
 
 		*newp = 0;
@@ -953,10 +1044,10 @@ static int pop3_getrange(int sock,
 
 		    if (parseuid(buf, &unum, id, sizeof(id)) == PS_SUCCESS)
 		    {
-			struct idlist	*old, *new;
+			struct idlist	*old, *newl;
 
-			new = save_str(&ctl->newsaved, id, UID_UNSEEN);
-			new->val.status.num = unum;
+			newl = save_str(&ctl->newsaved, id, UID_UNSEEN);
+			newl->val.status.num = unum;
 
 			if ((old = str_in_list(&ctl->oldsaved, id, FALSE)))
 			{
@@ -972,7 +1063,7 @@ static int pop3_getrange(int sock,
 				/* just mark it as seen now! */
 				old->val.status.mark = mark = UID_SEEN;
 			    }
-			    new->val.status.mark = mark;
+			    newl->val.status.mark = mark;
 			    if (mark == UID_UNSEEN)
 			    {
 				(*newp)++;
@@ -990,13 +1081,14 @@ static int pop3_getrange(int sock,
 			     * the same mail will not be downloaded again.
 			     */
 			    old = save_str(&ctl->oldsaved, id, UID_UNSEEN);
-			    old->val.status.num = unum;
 			}
+			/* save the number */
+			old->val.status.num = unum;
 		    } else
 			return PS_ERROR;
-		}
-	    }
-	}
+		} /* multi-line loop for UIDL reply */
+	    } /* UIDL parser */
+	} /* do UIDL */
     }
 
     return(PS_SUCCESS);
@@ -1005,15 +1097,15 @@ static int pop3_getrange(int sock,
 static int pop3_getpartialsizes(int sock, int first, int last, int *sizes)
 /* capture the size of message #first */
 {
-    int	ok = 0, i;
+    int	ok = 0, i, num;
     char buf [POPBUFSIZE+1];
-    unsigned int num, size;
+    unsigned int size;
 
     for (i = first; i <= last; i++) {
 	gen_send(sock, "LIST %d", i);
 	if ((ok = pop3_ok(sock, buf)) != 0)
 	    return(ok);
-	if (sscanf(buf, "%u %u", &num, &size) == 2) {
+	if (sscanf(buf, "%d %u", &num, &size) == 2) {
 	    if (num == i)
 		sizes[i - first] = size;
 	    else
@@ -1045,7 +1137,7 @@ static int pop3_getsizes(int sock, int count, int *sizes)
 	    if (DOTLINE(buf))
 		break;
 	    else if (sscanf(buf, "%u %u", &num, &size) == 2) {
-		if (num > 0 && num <= count)
+		if (num > 0 && num <= (unsigned)count)
 		    sizes[num - 1] = size;
 		else
 		    /* warn about possible attempt to induce buffer overrun */
@@ -1060,7 +1152,7 @@ static int pop3_getsizes(int sock, int count, int *sizes)
 static int pop3_is_old(int sock, struct query *ctl, int num)
 /* is the given message old? */
 {
-    struct idlist *new;
+    struct idlist *newl;
     if (!ctl->oldsaved)
 	return (num <= last);
     else if (dofastuidl)
@@ -1072,30 +1164,30 @@ static int pop3_is_old(int sock, struct query *ctl, int num)
 
 	/* in fast uidl, we manipulate the old list only! */
 
-	if ((new = id_find(&ctl->oldsaved, num)))
+	if ((newl = id_find(&ctl->oldsaved, num)))
 	{
 	    /* we already have the id! */
-	    return(new->val.status.mark != UID_UNSEEN);
+	    return(newl->val.status.mark != UID_UNSEEN);
 	}
 
 	/* get the uidl first! */
 	if (pop3_getuidl(sock, num, id, sizeof(id)) != PS_SUCCESS)
 	    return(TRUE);
 
-	if ((new = str_in_list(&ctl->oldsaved, id, FALSE))) {
+	if ((newl = str_in_list(&ctl->oldsaved, id, FALSE))) {
 	    /* we already have the id! */
-	    new->val.status.num = num;
-	    return(new->val.status.mark != UID_UNSEEN);
+	    newl->val.status.num = num;
+	    return(newl->val.status.mark != UID_UNSEEN);
 	}
 
 	/* save it */
-	new = save_str(&ctl->oldsaved, id, UID_UNSEEN);
-	new->val.status.num = num;
+	newl = save_str(&ctl->oldsaved, id, UID_UNSEEN);
+	newl->val.status.num = num;
 	return(FALSE);
     }
     else
-        return ((new = id_find(&ctl->newsaved, num)) != NULL &&
-	    new->val.status.mark != UID_UNSEEN);
+        return ((newl = id_find(&ctl->newsaved, num)) != NULL &&
+	    newl->val.status.mark != UID_UNSEEN);
 }
 
 #ifdef UNUSED
@@ -1129,8 +1221,9 @@ static int pop3_fetch(int sock, struct query *ctl, int number, int *lenp)
 
 #ifdef SDPS_ENABLE
     /*
-     * See http://www.demon.net/services/mail/sdps-tech.html
+     * See http://www.demon.net/helpdesk/producthelp/mail/sdps-tech.html/
      * for a description of what we're parsing here.
+     * -- updated 2006-02-22
      */
     if (ctl->server.sdps)
     {
@@ -1148,6 +1241,11 @@ static int pop3_fetch(int sock, struct query *ctl, int number, int *lenp)
 	    switch (linecount) {
 	    case 4:
 		/* No need to wrap envelope from address */
+		/* FIXME: some parts of fetchmail don't handle null
+		 * envelope senders, so use <> to mark null sender
+		 * as a workaround. */
+		if (strspn(buf, " \t") == strlen(buf))
+		    strcpy(buf, "<>");
 		sdps_envfrom = xmalloc(strlen(buf)+1);
 		strcpy(sdps_envfrom,buf);
 		break;
@@ -1160,6 +1258,8 @@ static int pop3_fetch(int sock, struct query *ctl, int number, int *lenp)
 	} while
 	    (!(buf[0] == '.' && (buf[1] == '\r' || buf[1] == '\n' || buf[1] == '\0')));
     }
+#else
+    (void)ctl;
 #endif /* SDPS_ENABLE */
 
     /*
@@ -1234,6 +1334,7 @@ static int pop3_delete(int sock, struct query *ctl, int number)
 static int pop3_mark_seen(int sock, struct query *ctl, int number)
 /* mark a given message as seen */
 {
+    (void)sock;
     mark_uid_seen(ctl, number);
     return(PS_SUCCESS);
 }
@@ -1271,8 +1372,8 @@ static int pop3_logout(int sock, struct query *ctl)
 static const struct method pop3 =
 {
     "POP3",		/* Post Office Protocol v3 */
-    "pop3",		/* standard POP3 port */
-    "pop3s",		/* ssl POP3 port */
+    "pop3",		/* port for plain and TLS POP3 */
+    "pop3s",		/* port for SSL POP3 */
     FALSE,		/* this is not a tagged protocol */
     TRUE,		/* this uses a message delimiter */
     pop3_ok,		/* parse command response */
